/* /////////////////////////////////////////////////////////////////////////////
 * File:        pathchk.cpp
 *
 * Purpose:     Implementation file for the pathchk project.
 *
 * Created:     28th April 2003
 * Updated:     21st May 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>

#include <stlsoft.h>

#include <stlsoft/string_tokeniser.hpp>

stlsoft_ns_using(string_tokeniser)
stlsoft_ns_using(string_tokeniser_ignore_blanks)
stlsoft_ns_using(string_tokeniser_type_traits)
stlsoft_ns_using(string_tokeniser_comparator)

#include <stlsoft/string_view.hpp>

stlsoft_ns_using(basic_string_view)

#include <stlsoft/simple_string.hpp>

stlsoft_ns_using(basic_simple_string)

#include <platformstl/platformstl.hpp>

#if defined(PLATFORMSTL_OS_IS_UNIX)

# include <unixstl.h>

# include <unixstl/environment_variable.hpp>

using unixstl::basic_environment_variable;

# include <unixstl/filesystem_traits.hpp>

typedef unixstl::filesystem_traits<char>	filesystem_traits_t;

#elif defined(PLATFORMSTL_OS_IS_WIN32)

# include <winstl.h>
# include <winstl/environment_variable.hpp>

using winstl::basic_environment_variable;

# include <winstl/filesystem_traits.hpp>

typedef winstl::filesystem_traits<char>		filesystem_traits_t;

#else
# error Platform not discriminated
#endif /* platform */

#include <stlsoft/string_access.hpp>

#include <algorithm>

using std::for_each;

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef _SYNSOFT_INTERNAL_BUILD
# include <MLFlAttr.h>					// FILE_ATTRIBUTE_ERROR
# include <MLCompId.h>					// The _xxxSynesisSoftware constants
#else /* ? _SYNSOFT_INTERNAL_BUILD */
# ifndef FILE_ATTRIBUTE_ERROR
#  define FILE_ATTRIBUTE_ERROR			(0xffffffff)
# endif /* FILE_ATTRIBUTE_ERROR */
# define _nameSynesisSoftware				"Synesis Software (Pty) Ltd"
# define _wwwSynesisSoftware_SystemTools	"http://synesis.com.au/"
# define _emailSynesisSoftware_SystemTools	"software@synesis.com.au"
#endif /* _SYNSOFT_INTERNAL_BUILD */

/* /////////////////////////////////////////////////////////////////////////////
 * Warnings
 */

#ifdef __STLSOFT_COMPILER_IS_INTEL
# pragma warning(disable : 981)
#endif /* __STLSOFT_COMPILER_IS_INTEL */

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

static void usage(int bExit, char const *reason);

typedef basic_simple_string<char>					string_t;
typedef basic_string_view<char>						string_view_t;
#if 0
typedef	string_tokeniser<string_t, char>			tokeniser_t;
#else
typedef	string_tokeniser<	string_t
						,	char
						,	string_tokeniser_ignore_blanks<true>
						,	string_view_t
						,	string_tokeniser_type_traits<string_t, string_view_t>
						,	string_tokeniser_comparator<	char
														,	string_t
														,	string_tokeniser_type_traits<string_t, string_view_t>
														>
						>							tokeniser_t;
#endif /* 0 */
typedef	basic_environment_variable<char>			envvar_t;

/* /////////////////////////////////////////////////////////////////////////////
 * Version information
 *
 * Normally, Synesis Software components & tools are "versioned" by making use
 * of tool-generated header files, one of which - MBldHdr.h - defines symbolic
 * constants representing major, minor, revision and build numbers.
 *
 *
 * Since this file is to be available in a more public forum, including on the
 * Digital Mars site, these constants are merely defined here, rather than
 * encumber any developers who may wish to compile it with a lot of
 * Synesis-specific gunk.
 */

#ifdef _SYNSOFT_INTERNAL_BUILD
# include "MBldHdr.h"
# include "showver.h"

extern "C" const int	_mccVerHi	=	__SYV_MAJOR;
extern "C" const int	_mccVerLo	=	__SYV_MINOR;
extern "C" const int	_mccVerRev	=	__SYV_REVISION;
extern "C" const int	_mccBldNum	=	__SYV_BUILDNUMBER;

#else /* ? _SYNSOFT_INTERNAL_BUILD */
extern "C" const int	_mccVerHi	=	1;
extern "C" const int	_mccVerLo	=	1;
extern "C" const int	_mccVerRev	=	1;
extern "C" const int	_mccBldNum	=	3;
#endif /* _SYNSOFT_INTERNAL_BUILD */

#define	COMPANY_NAME	"Synesis Software"
#define	TOOL_NAME		"Operating System Version Detection"
#define	EXE_NAME		"osver"

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

#if defined(PLATFORMSTL_OS_IS_UNIX)
const char	DELIMITER	=	':';
#elif defined(PLATFORMSTL_OS_IS_WIN32)
const char	DELIMITER	=	';';
#else
# error Platform not discriminated
#endif /* platform */

/* ////////////////////////////////////////////////////////////////////////// */

struct validate_file
{
	template<ss_typename_param_k S>
	void operator ()(S const &s)
	{
		if(!filesystem_traits_t::file_exists(stlsoft::c_str_ptr(s)))
		{
			fprintf(stdout, "\"%s\" does not exist\n", stlsoft::c_str_ptr(s));
		}
	}

	/* VC5 doesn't like basic_static_string's CCH parameter, so we need to provide
	 * an explicit operator() here
	 */
	void operator ()(string_view_t const &s)
	{
		operator ()(s.c_str());
	}
};

/* ////////////////////////////////////////////////////////////////////////// */

int main(int argc, char **argv)
{
#ifdef _DEBUG
	puts("pathchk: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* _DEBUG */

	if(argc > 1)
	{
		char const	*arg	=	argv[1];

		if(0 == strcmp("-?", argv[1]) ||
#ifdef SYSTOOLS_ALLOW_SLASH_OPTIONS
				0 == strcmp("/?", argv[1]) ||
#endif /* SYSTOOLS_ALLOW_SLASH_OPTIONS */
				0)
		{
			usage(1, NULL);
		}
#ifdef _SYNSOFT_INTERNAL_BUILD
		else if(0 == strcmp("--version", arg))
		{
			return show_version();
		}
#endif /* _SYNSOFT_INTERNAL_BUILD */
		else
		{
			usage(1, "Unrecognised argument");
		}
	}
	else
	{
		// Retrive and tokenise the PATH environment variable ...
		envvar_t	PATHS("PATH");
		tokeniser_t	tokens(PATHS, DELIMITER);

		// ... and then print out any that do not exist
		for_each(tokens.begin(), tokens.end(), validate_file());
	}

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */

void usage(int bExit, char const *reason)
{
	FILE	*stm	=	(NULL == reason) ? stdout : stderr;

    fprintf(stm, "\n");
	fprintf(stm, "  %s %s Tool, v%d.%d.%d.%04d\n", COMPANY_NAME, TOOL_NAME, _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stm, "\n");

	if(NULL != reason)
	{
		fprintf(stm, "  Error: %s\n", reason);
		fprintf(stm, "\n");
	}

	fprintf(stm, "  USAGE 1: %s\n", EXE_NAME);
    fprintf(stm, "\n");
	fprintf(stm, "    Lists all invalid components of the system PATH\n");
	fprintf(stm, "    environment variable\n");
    fprintf(stm, "\n");
    fprintf(stm, "  USAGE 2: %s -?\n", EXE_NAME);
    fprintf(stm, "\n");
    fprintf(stm, "    Displays this help\n");
    fprintf(stm, "\n");
	fprintf(stm, " Contact %s\n", _nameSynesisSoftware);
	fprintf(stm, "   at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stm, "   or via email at \"%s\"\n", _emailSynesisSoftware_SystemTools);
	fprintf(stm, "\n");

	if(bExit)
	{
		exit(EXIT_FAILURE);
	}
}

/* ////////////////////////////////////////////////////////////////////////// */
