/* /////////////////////////////////////////////////////////////////////////////
 * File:        socket_test.cpp
 *
 * Purpose:     Implementation file for the socket_test project.
 *
 * Created:     22nd March 2005
 * Updated:     22nd March 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>

#include <platformstl/platformstl.h>
#include <platformstl/platformstl.hpp>
#include <platformstl/atomic_functions.h>
#include <platformstl/platform_exceptions.hpp>

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h>

/* Standard C++ Header Files */
#include <exception>

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv);

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

namespace inetstl
{
	using ::stlsoft::ss_bool_t;

	namespace implementation
	{
		typedef stlsoft::ss_sint32_t	rc_t;

	} // namespace implementation

	struct refcount_policy_single_threaded
	{
	public:
		static void addref(implementation::rc_t &cRefs)
		{
			++cRefs;
		}
		static implementation::rc_t release(implementation::rc_t &cRefs)
		{
			return --cRefs;
		}
	};

	struct refcount_policy_multi_threaded
	{
	public:
		static void addref(implementation::rc_t &cRefs)
		{
			platformstl::atomic_increment(&cRefs);
		}
		static implementation::rc_t release(implementation::rc_t &cRefs)
		{
			return platformstl::atomic_predecrement(&cRefs);
		}
	};

	template <typename R = refcount_policy_multi_threaded>
	class socket_handle
	{
	public:
#if defined(WIN32)
		typedef SOCKET				handle_type;
#else /* ? WIN32 */
		typedef int					handle_type;
#endif /* WIN32 */
		typedef	R					refcount_policy;
		typedef socket_handle<R>	class_type;

	public:
		class Ref
		{
		private:
			typedef socket_handle<R>	Handle_type;
		public:
			Ref(Handle_type *ph)
				: m_ph(ph)
			{}
			~Ref()
			{
				if(NULL != m_ph)
				{
					m_ph->Release();
				}
			}
			Ref(Ref const &rhs)
				: m_ph(rhs.m_ph)
			{
				if(NULL != m_ph)
				{
					m_ph->AddRef();
				}
			}
			Ref &operator =(Ref const &rhs)
			{
				if(NULL != m_ph)
				{
					m_ph->Release();
				}

				m_ph = rhs.m_ph;

				if(NULL != m_ph)
				{
					m_ph->AddRef();
				}

				return *this;
			}

			Handle_type *operator ->()
			{
				stlsoft_message_assert("instance does not have a non-NULL handle", NULL != m_ph);

				return m_ph;
			}
			Handle_type const *operator ->() const
			{
				stlsoft_message_assert("instance does not have a non-NULL handle", NULL != m_ph);

				return m_ph;
			}

		private:
			Handle_type	*m_ph;
		};

	public:
		handle_type				h;
		implementation::rc_t	cRefs;
		const ss_bool_t			bOwn;

		socket_handle(handle_type h_, ss_bool_t bOwn_)
			: h(h_)
			, cRefs(1)
			, bOwn(bOwn_)
		{}

		socket_handle *AddRef()
		{
			refcount_policy::addref(cRefs);

			return this;
		}

		void Release()
		{
			if(0 == refcount_policy::release(cRefs))
			{
				if(bOwn)
				{
					Destroy(h);
				}

				delete this;
			}
		}

	public:
		static void Destroy(handle_type h)
		{
			if(get_null_value() != h)
			{
#if defined(WIN32)
				closesocket(h);
#else /* ? WIN32 */
				close(h);
#endif /* WIN32 */
			}
		}

	public:
		static handle_type get_null_value()
		{
#if defined(WIN32)
			return INVALID_SOCKET;
#else /* ? WIN32 */
			return static_cast<handle_type>(-1);
#endif /* WIN32 */
		}
	};

	typedef socket_handle<>		HSocket_t;

	/// socket
	class socket
	{
	public:
#if defined(WIN32)
		typedef SOCKET	handle_type;
#else /* ? WIN32 */
		typedef int		handle_type;
#endif /* WIN32 */
		typedef socket	class_type;

	/// \name Construction
	/// @{
	public:
		/// Default constructor
		socket();
		socket(HSocket_t::Ref h);
		socket(int addressFamily, int type, int protocol);

		socket(class_type const &rhs);

		~socket();
	/// @}

	/// \name Operations
	/// @{
	public:
		HSocket_t::Ref accept();
	/// @}

	/// \name Accessors
	/// @{
	public:
		handle_type get_handle() const
		{
			return m_handle->h;
		}
	/// @}

	/// Implementation
	private:
		static HSocket_t::Ref	open_socket_(	int addressFamily
											,	int type
											,	int	protocol);

	/// Members
	private:
		HSocket_t::Ref	m_handle;
	};


} // namespace inetstl

/* ////////////////////////////////////////////////////////////////////////// */

int main_(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int                 iRet    =   0;
	int					bVerbose	=	true;
    int                 i;

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	puts("socket_test: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
				switch(arg[1])
				{
					case	's':
						bVerbose	=	false;
						break;
					case	'v':
						bVerbose	=	true;
						break;
					default:
						usage(1, "Invalid argument(s) specified", i, argc, argv);
						break;
				}
			}
		}
		else
		{
			/* other arguments */
			usage(1, "Invalid argument(s) specified", i, argc, argv);
		}
	}

	/* . */
	WSADATA	wsaData;
	int		r	=	::WSAStartup(0x0202, &wsaData);

	::inetstl::socket	socket(AF_INET, SOCK_STREAM, 0);

//	::bind(

	::inetstl::socket	socket2	=	socket.accept();

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv)
{

    fprintf(stderr, "\n");
    fprintf(stderr, "  socket_test\n\n");
    fprintf(stderr, "\n");

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n", reason);
		fprintf(stderr, "\n");
	}

#if 0
    fprintf(stderr, "  USAGE: socket_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    -w             - searches from the current working directory. The default\n");
    fprintf(stderr, "    -p<root-paths> - searches from the given root path(s), separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       -p\"c:\\windows;x:\\bin\"\n");
    fprintf(stderr, "    -r             - deletes readonly files\n");
    fprintf(stderr, "    -h             - searches from the roots of all drives on the system\n");
    fprintf(stderr, "    -d             - displays the path(s) searched\n");
    fprintf(stderr, "    -u             - do not act recursively\n");
    fprintf(stderr, "    -v             - verbose output. Prints time, attributes, size and path. (default)\n");
    fprintf(stderr, "    -s             - succinct output. Prints path only\n");
    fprintf(stderr, "    <search-spec>  - one or more file search specifications, separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       \"*.exe\"\n");
    fprintf(stderr, "                       \"myfile.ext\"\n");
    fprintf(stderr, "                       \"*.exe;*.dll\"\n");
    fprintf(stderr, "                       \"*.xl?;report.*\"\n");
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");
#endif /* 0 */

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		fprintf(stderr, "Unhandled error: %s\n", x.what());
	}
	catch(...)
	{
		fprintf(stderr, "Unhandled unknown error\n");
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */

namespace inetstl
{
	/* static */ HSocket_t::Ref socket::open_socket_(	int addressFamily
													,	int type
													,	int	protocol)
	{
		HSocket_t::handle_type	h	=	::socket(addressFamily, type, protocol);

		if(HSocket_t::get_null_value() == h)
		{
			throw ::platformstl::platform_exception(::WSAGetLastError());
		}

		try
		{
			return HSocket_t::Ref(new HSocket_t(h, true));
		}
		catch(std::exception &)
		{
			HSocket_t::Destroy(h);

			throw;
		}
	}

	socket::socket(HSocket_t::Ref h)
		: m_handle(h)
	{}

	socket::socket()
		: m_handle(NULL)
	{}

	socket::socket(int addressFamily, int type, int protocol)
		: m_handle(open_socket_(addressFamily, type, protocol))
	{}
	
	socket::~socket()
	{}

	HSocket_t::Ref socket::accept()
	{
		HSocket_t::handle_type	h	=	::accept(m_handle->h, NULL, 0);

		if(HSocket_t::get_null_value() == h)
		{
			throw ::platformstl::platform_exception("accept failed", ::WSAGetLastError());
		}

		try
		{
			return HSocket_t::Ref(new HSocket_t(h, true));
		}
		catch(std::exception &)
		{
			HSocket_t::Destroy(h);

			throw;
		}
	}

} // namespace inetstl

/* ////////////////////////////////////////////////////////////////////////// */
