/* /////////////////////////////////////////////////////////////////////////////
 * File:        array_proxy.cpp
 *
 * Purpose:     Implementation file for the array_proxy project.
 *
 * Created:     28th July 2003
 * Updated:     30th May 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#define	_STLSOFT_COMPILE_VERBOSE

#include <stdio.h>

#include <stlsoft.h>
#include <stlsoft_array_proxy.h>

stlsoft_ns_using(array_proxy)
stlsoft_ns_using(make_array_proxy)

////////////////////////////////////////////////////////////////////////////////

struct Root
{
	int	i;

	Root(int i = 0)
		: i(i)
	{}
};

struct Parent
	: public Root
{
	Parent(int i = 1)
		: Root(i)
	{}
};

struct Derived_Bigger
	: public Parent
{
	int	j;

	Derived_Bigger(int i = 1)
		: Parent(i)
		, j(i + 1)
	{}
};

struct Derived_Same
	: public Parent
{
	Derived_Same(int i = 1)
		: Parent(i)
	{}
};

struct Independent_Same
{
	int	i;

	Independent_Same(int i = 1)
		: i(i)
	{
		stlsoft_static_assert(sizeof(Independent_Same) == sizeof(Parent));
	}
};


void process_parent(Parent const *begin, Parent const *end)
{
	printf("\n");
	for(; begin != end; ++begin)
	{
		printf("%d ", begin->i);
	}
	printf("\n");
}

void process_derived(Derived_Bigger const *begin, Derived_Bigger const *end)
{
	printf("\n");
	for(; begin != end; ++begin)
	{
		printf("[%d, %d] ", begin->i, begin->j);
	}
	printf("\n");
}

void process(array_proxy<Parent> const &proxy)
{
	process_parent(&proxy[0], &proxy[proxy.size()]);
//	process_parent(proxy.begin(), proxy.end());
}

#if 0
void process(array_proxy<Derived_Bigger> const &proxy)
{
	process_derived(proxy.begin(), proxy.end());
}
#endif /* 0 */

template <ss_typename_param_k T>
void process(T const *begin, T const *end)
{
	process_parent(begin, end);
}

/* ////////////////////////////////////////////////////////////////////////// */

int main(int /* argc */, char ** /*argv*/)
{
	/* . */
	stlsoft_static_assert(sizeof(Parent) != sizeof(Derived_Bigger));

	Parent				parents[10];
	Derived_Bigger		bigger[10];
	Derived_Same		same[10];
	Root				roots[10];
	Independent_Same	independent[10];

	process(parents, parents + stlsoft_num_elements(parents));
	process(bigger, bigger + stlsoft_num_elements(bigger));
	process(same, same + stlsoft_num_elements(same));

#if defined(__STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT) && \
	/* !defined(__STLSOFT_COMPILER_IS_DMC) && */ \
	/* !defined(__STLSOFT_COMPILER_IS_MWERKS) && */ \
	(	!defined(__STLSOFT_COMPILER_IS_MSVC) || \
		_MSC_VER >= 1310)
	process(array_proxy<Parent>(parents));
//	process(array_proxy<Derived_Bigger>(bigger));
	process(array_proxy<Parent>(same));
#endif /* __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT */

#if defined(__STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT) && \
	(	!defined(__STLSOFT_COMPILER_IS_MSVC) || \
		_MSC_VER >= 1310)
	process(make_array_proxy(parents));
//	process(make_array_proxy(bigger));
	process(make_array_proxy(same));
#endif /* __STLSOFT_CF_STATIC_ARRAY_SIZE_DETERMINATION_SUPPORT */

	typedef void (*pfn)(array_proxy<Parent> const &);

	pfn	p	=	process;

	process(make_array_proxy(&parents[0], &parents[stlsoft_num_elements(parents)]));
//	process(make_array_proxy(&bigger[0], &bigger[stlsoft_num_elements(bigger)]));
	(*p)(make_array_proxy(&same[0], &same[stlsoft_num_elements(same)]));
//	(*p)(make_array_proxy(&bigger[0], &bigger[stlsoft_num_elements(bigger)]));
//	(*p)(make_array_proxy(&roots[0], &roots[stlsoft_num_elements(roots)]));

	process(make_array_proxy(&parents[0], stlsoft_num_elements(parents)));
//	process(make_array_proxy(&bigger[0], stlsoft_num_elements(bigger)));
	(*p)(make_array_proxy(&same[0], stlsoft_num_elements(same)));

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
