/* /////////////////////////////////////////////////////////////////////////////
 * File:        bit_int_test.cpp
 *
 * Purpose:     Implementation file for the bit_int_test project.
 *
 * Created:     17th October 2003
 * Updated:     30th May 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>

#include <stlsoft.h>
#include <stlsoft_bit_int.h>

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef STLSOFT_CF_64BIT_INT_SUPPORT

struct integer64
{
	ss_uint32_t	lowerVal;
	ss_int32_t	upperVal;
};

struct uinteger64
{
	ss_uint32_t	lowerVal;
	ss_uint32_t	upperVal;
};

/* 64-bit */
#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct bit_int<64>
{
	enum { bits = 64 };

	typedef integer64	signed_type;
	typedef uinteger64	unsigned_type;
};


#endif /* STLSOFT_CF_64BIT_INT_SUPPORT */

/* ////////////////////////////////////////////////////////////////////////// */

template <ss_typename_param_k T>
struct test_integral_type
{
	void operator ()() const
	{
//		{

		T	t1;
		T	t2	=	0;
		T	t3	=	T();
		T	t4	=	T(0);
		T	t5	=	t2;

		t1 = t2;

	}
};

template <ss_typename_param_k T>
void run_test(T t)
{
	t();
}

/* ////////////////////////////////////////////////////////////////////////// */

typedef stlsoft_ns_qual(bit_int)<8>::signed_type		sint8_t;
typedef stlsoft_ns_qual(bit_int)<8>::unsigned_type		uint8_t;
typedef stlsoft_ns_qual(bit_int)<16>::signed_type		sint16_t;
typedef stlsoft_ns_qual(bit_int)<16>::unsigned_type		uint16_t;
typedef stlsoft_ns_qual(bit_int)<32>::signed_type		sint32_t;
typedef stlsoft_ns_qual(bit_int)<32>::unsigned_type		uint32_t;
typedef stlsoft_ns_qual(bit_int)<64>::signed_type		sint64_t;
typedef stlsoft_ns_qual(bit_int)<64>::unsigned_type		uint64_t;


int main(int /* argc */, char ** /*argv*/)
{
	puts("bit_int_test: " __STLSOFT_COMPILER_LABEL_STRING);

	sint8_t		s8;
	uint8_t		u8;
	sint16_t	s16;
	uint16_t	u16;
	sint32_t	s32;
	uint32_t	u32;
	sint64_t	s64;
	uint64_t	u64;

	stlsoft_static_assert(sizeof(s8) == 8 / 8);
	stlsoft_static_assert(sizeof(u8) == 8 / 8);
	stlsoft_static_assert(sizeof(s16) == 16 / 8);
	stlsoft_static_assert(sizeof(u16) == 16 / 8);
	stlsoft_static_assert(sizeof(s32) == 32 / 8);
	stlsoft_static_assert(sizeof(u32) == 32 / 8);
#ifdef STLSOFT_CF_64BIT_INT_SUPPORT
	stlsoft_static_assert(sizeof(s64) == 64 / 8);
	stlsoft_static_assert(sizeof(u64) == 64 / 8);
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

	STLSOFT_SUPPRESS_UNUSED(s8);
	STLSOFT_SUPPRESS_UNUSED(u8);
	STLSOFT_SUPPRESS_UNUSED(s16);
	STLSOFT_SUPPRESS_UNUSED(u16);
	STLSOFT_SUPPRESS_UNUSED(s32);
	STLSOFT_SUPPRESS_UNUSED(u32);
#ifdef STLSOFT_CF_64BIT_INT_SUPPORT
	STLSOFT_SUPPRESS_UNUSED(s64);
	STLSOFT_SUPPRESS_UNUSED(u64);
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

	run_test(test_integral_type<sint8_t>());
	run_test(test_integral_type<uint8_t>());
	run_test(test_integral_type<sint16_t>());
	run_test(test_integral_type<uint16_t>());
	run_test(test_integral_type<sint32_t>());
	run_test(test_integral_type<uint32_t>());
	run_test(test_integral_type<sint64_t>());
	run_test(test_integral_type<uint64_t>());

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
