/* /////////////////////////////////////////////////////////////////////////////
 * File:        string_view_test.cpp
 *
 * Purpose:     Implementation file for the string_view_test project.
 *
 * Created:     8th February 2005
 * Updated:     9th June 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>
#include <stlsoft/simple_string.hpp>
#include <stlsoft/string_view.hpp>

/* Standard C Header Files */

/* Standard C++ Header Files */
#include <exception>
#include <iostream>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

#if (   defined(WIN32) || \
        defined(_WIN32)) && \
    !defined(_INC_WINDOWS)
extern "C" void __stdcall Sleep(unsigned long);
#endif /* WIN32 */


/* ////////////////////////////////////////////////////////////////////////// */

static int main_(int /* argc */, char ** /*argv*/)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "string_view_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	/* . */
	typedef ::stlsoft::simple_string			string_t;
	typedef ::stlsoft::basic_string_view<char>	string_view_t;


	string_t		s1("Humpty Dumpty sat on a wall");

	string_view_t	sv1(&s1[7], 6);

//	string_view_t	sv1(&s1[7], 6);

	if("Dumpty" == sv1)
	{
		cout << "Found: " << sv1 << endl;
	}

	char const	*dumpty	=	sv1.c_str();

	{
        string_t        s2("String 2");

        string_view_t   sv2(s2.data(), s2.length());

        stlsoft_assert(sv2 == "String 2");
        stlsoft_assert(0 == strcmp(sv2.c_str(), "String 2"));
        stlsoft_assert(0 == strncmp(sv2.data(), "String 2", sv2.length()));

        s2.back() = '3';

        stlsoft_assert(sv2 == "String 3");
        stlsoft_assert(0 == strcmp(sv2.c_str(), "String 2"));
        stlsoft_assert(0 == strncmp(sv2.data(), "String 3", sv2.length()));

        sv2.refresh();

        stlsoft_assert(sv2 == "String 3");
        stlsoft_assert(0 == strcmp(sv2.c_str(), "String 3"));
        stlsoft_assert(0 == strncmp(sv2.data(), "String 3", sv2.length()));
	}

    return 0;
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
